unit ufmZmeter;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  TeEngine, Series, ExtCtrls, TeeProcs, Chart, ufrSpRange, StdCtrls,
  ufrWaveSetup, ComCtrls, Menus,
  uSignalIn, 
  uWaveOut, uWaveControl, ufrValues;

type
  enShowMode = (smSamples, smFFTCom1, smFFTCom2, smFFTMod, smFFTModRatio);
  TForm1 = class(TForm)
    PageControl1: TPageControl;
    tsSetup: TTabSheet;
    tsProcessing: TTabSheet;
    Chart: TChart;
    Series1: TFastLineSeries;
    Series2: TFastLineSeries;
    frSpRange1: TfrSpRange;
    frWaveSetup1: TfrWaveSetup;
    btnStartStop: TButton;
    Series3: TFastLineSeries;
    Series4: TFastLineSeries;
    gbGraph: TGroupBox;
    cbModPh: TCheckBox;
    cbDrawAll: TCheckBox;
    cbLeftForAll: TCheckBox;
    cbLogF: TCheckBox;
    gbCalibration: TGroupBox;
    lbCalibration: TLabel;
    btnCalibration: TButton;
    lbSaturation: TLabel;
    btnSave: TButton;
    gbLCR: TGroupBox;
    memLCRRes: TMemo;
    rgMode: TRadioGroup;
    pnRref: TPanel;
    Label1: TLabel;
    frRref: TfrValues;
    lbAutor: TLabel;
    Image1: TImage;
    cbIntegration: TComboBox;
    Label2: TLabel;
    procedure btnStartStopClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cbDrawAllClick(Sender: TObject);
    procedure cbLeftForAllClick(Sender: TObject);
    procedure cbLogFClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnCalibrationClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure ChartClickAxis(Sender: TCustomChart; Axis: TChartAxis;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure rgModeClick(Sender: TObject);
    procedure cbModPhClick(Sender: TObject);
    procedure cbIntegrationClick(Sender: TObject);
  private
    { Private declarations }
    FDecimalSeparator: Char;
    FRunning: Boolean;
    FInput: TSignalIn;
    FOutput: TWaveOut;
    FSeries: array[0..3] of TFastLineSeries;
    FSeriesNumber: Integer;
    FCount: Integer;
    FMin, FMax: Double;
    FMode: enShowMode;
    FAmp: Double;
    FCurMesId: Integer;
    FAmPh: Boolean;
    procedure LoadIni;
    procedure SaveIni;
    procedure SetGraphParams;
    procedure AllowData(Sender: TObject);
    procedure ShowData(aData: TWaveData);
    procedure DataProcess(aData: TWaveData);
    procedure SpRangeChange(aMin, aMax: Double);
    procedure OnSamplingChange(aSampling: DWORD);
    procedure ShowErrorMessage(const aMessage: String);
    procedure ShowCalibMesssage(const aMessage: String);
    procedure ShowLCMessage(const aMessage: String);
    procedure DrawAll(aValue: Boolean);
    procedure LeftForAll(aValue: Boolean);
    procedure ClearSeries;
    procedure Saturation(aValue: Boolean);
    procedure SetAmp(aValue: Double);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Math, uFFT, IniFiles, uCom;               

const
  csGraphSection = 'Graph';
  cnMinFreq = 20;
  cnMaxFreq = 22000;

procedure TForm1.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  lbAutor.Caption:= lbAutor.Caption + Chr(13) + Chr(10) +
    'http://deforg.free.fr/' + Chr(13) + Chr(10) +
    'dforgeron@mail.ru';
  FDecimalSeparator:= DecimalSeparator;
  DecimalSeparator:= '.';
  PageControl1.ActivePageIndex:= 0;
  FCurMesId:= 2;
  FOutput:= TWaveOut.Create;
  FInput:= TSignalIn.Create;
  FOutput.Validate:= FInput.Validation;
  FInput.ErrorMes:= ShowErrorMessage;
  FOutput.ErrorMes:= ShowErrorMessage;
  FRunning:= False;
  FSeries[0]:= Series1;
  FSeries[1]:= Series2;
  FSeries[2]:= Series3;
  FSeries[3]:= Series4;
  for i:= 0 to 3 do
    FSeries[i].AfterDrawValues:= AllowData;
  frWaveSetup1.ProcSampling:= OnSamplingChange;
  FInput.WaveProc:= DataProcess;
  frSpRange1.RangeProc:= SpRangeChange;
  frRref.CurrentValue:= 1;
  frRref.SectionName:= 'Rref';
  LoadIni;
  SetGraphParams;
  OnSamplingChange(frWaveSetup1.Sampling);
  rgModeClick(nil);
  FInput.OnSaturation:= Saturation;
  FInput.SetAmp:= SetAmp;
  Chart.RightAxis.Grid.Visible:= False;
  FAmp:= 1;
  cbIntegration.ItemIndex:= 0;
  cbIntegrationClick(nil);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FInput.Free;
  FOutput.Free;
  DecimalSeparator:= FDecimalSeparator;
end;

procedure TForm1.LoadIni;
var
  aFile: TIniFile;
begin
  aFile:= TIniFile.Create(ChangeFileExt(Paramstr(0),'.ini'));
  frWaveSetup1.LoadIni(aFile);
  frRref.LoadIni(aFile);
  FInput.LoadIni(aFile);
  cbModPh.Checked:= aFile.ReadBool(csGraphSection, 'ModPhase',
                           cbModPh.Checked);
  cbDrawAll.Checked:= aFile.ReadBool(csGraphSection, 'DrawAll',
                      cbDrawAll.Checked);
  cbLeftForAll.Checked:= aFile.ReadBool(csGraphSection, 'LeftForAll',
                         cbLeftForAll.Checked);
  cbLogF.Checked:= aFile.ReadBool(csGraphSection, 'Lg(F)',
                   cbLogF.Checked);
  PageControl1.ActivePageIndex:= aFile.ReadInteger(csGraphSection, 'Page',
                                 PageControl1.ActivePageIndex);
  rgMode.ItemIndex:= aFile.ReadInteger(csGraphSection, 'Mode', rgMode.ItemIndex);
  aFile.Free;
end;

procedure TForm1.SaveIni;
var
  aFile: TIniFile;
begin
  aFile:= TIniFile.Create(ChangeFileExt(Paramstr(0),'.ini'));
  frWaveSetup1.SaveIni(aFile);
  frRref.SaveIni(aFile);
  FInput.SaveIni(aFile);
  aFile.WriteBool(csGraphSection, 'ModPhase', cbModPh.Checked);
  aFile.WriteBool(csGraphSection, 'DrawAll', cbDrawAll.Checked);
  aFile.WriteBool(csGraphSection, 'LeftForAll', cbLeftForAll.Checked);
  aFile.WriteBool(csGraphSection, 'Lg(F)', cbLogF.Checked);
  aFile.WriteInteger(csGraphSection, 'Page', PageControl1.ActivePageIndex);
  aFile.WriteInteger(csGraphSection, 'Mode', rgMode.ItemIndex);
  aFile.Free;
end;

procedure TForm1.SetGraphParams;
begin
  FInput.Accumulation:= True;
  cbModPhClick(Self);
  cbDrawAllClick(Self);
  cbLeftForAllClick(Self);
  cbLogFClick(Self);
end;

procedure TForm1.btnStartStopClick(Sender: TObject);
const
  cnCalibrFreq = 440;
begin
  if FRunning then begin
    FInput.Stop;
    FOutput.Stop;
    FRunning:= False;
    btnStartStop.Caption:= 'Start';
    Saturation(False);
    if rgMode.ItemIndex = 0 then begin
      if not Chart.LeftAxis.Logarithmic then
        rgMode.ItemIndex:= FCurMesId;
      Chart.LeftAxis.Logarithmic:= False;
      SetGraphParams;
    end;
  end else begin
    FOutput.Device:= frWaveSetup1.DeviceOutId;
    FInput.Device:= frWaveSetup1.DeviceInId;
    FInput.Sampling:= frWaveSetup1.Sampling;
    FOutput.Sampling:= FInput.Sampling;
    FInput.DataCount:= FCount;
    case rgMode.ItemIndex of
      0:begin
          FAmp:= 1;
          FOutput.SetRange(cnCalibrFreq, cnCalibrFreq, FAmp, FCount);
          FInput.DoAdjustement(cnCalibrFreq, ShowCalibMesssage);
          FMode:= smFFTMod;
          ClearSeries;
          Chart.LeftAxis.Automatic:= True;
          Chart.BottomAxis.Automatic:= True;
          Chart.LeftAxis.Logarithmic:= True;
          Chart.BottomAxis.Logarithmic:= True;
          LeftForAll(True);
        end;
      else
        begin
          FOutput.SetRange(frSpRange1.Min, frSpRange1.Max, FAmp, FCount);
          FInput.DoAcquisition(rgMode.ItemIndex = 4);
          case rgMode.ItemIndex of
            2: FMode:= smFFTCom2;
            4: FMode:= smFFTCom2;
            else
              FMode:= smFFTCom1;
          end;
          Chart.LeftAxis.Logarithmic:= False;
        end;
    end;
    if FOutput.Start and FInput.Start then begin
      btnStartStop.Caption:= 'Stop';
      FRunning:= True;
    end else begin
      FInput.Stop;
      FOutput.Stop;
    end;
  end;
  frWaveSetup1.AllowInput(not FRunning);
  rgMode.Enabled:= not FRunning;
  btnCalibration.Visible:= FRunning;
end;

procedure TForm1.btnCalibrationClick(Sender: TObject);
begin
  ClearSeries;
  FOutput.SetRange(cnMinFreq, cnMaxFreq, FAmp, FCount);
  FInput.DoCalibration(cnMinFreq, cnMaxFreq);
  Chart.LeftAxis.Logarithmic:= False;
  Chart.BottomAxis.Logarithmic:= False;
  FMode:= smFFTCom2;
  LeftForAll(False);
  btnCalibration.Visible:= False;
  FInput.Accumulation:= True;
end;

procedure TForm1.SetAmp(aValue: Double);
begin
  if FAmp * aValue > 1 then
    if FAmp = 1 then
      Exit
    else
      FAmp:= 1
  else
    FAmp:= FAmp * aValue;
  if FAmp > 1 then
    FAmp:= 1;
  if rgMode.ItemIndex = 0 then
    FOutput.SetRange(cnMinFreq, cnMaxFreq, FAmp, FCount)
  else
    FOutput.SetRange(FMin, FMax, FAmp, FCount);
end;

procedure TForm1.AllowData(Sender: TObject);
begin
  Dec(FSeriesNumber);
  if FSeriesNumber = 0 then
    FInput.AllowData(Sender);
end;

procedure TForm1.ShowData(aData: TWaveData);
var
  i, j, aMin, aMax: Integer;
  aSampling: DWORD;
  aBaseFreq: Double;
  aVal: Com;
begin
  ClearSeries;
  FSeriesNumber:= 0;
  aSampling:= FInput.Sampling;
  if FMode = smFFTModRatio then begin
    FSeriesNumber:= 1;
    aBaseFreq:= aSampling/Length(aData[0]);
    aMin:= FOutput.GetHarmonic(FMin, FCount);
    aMax:= FOutput.GetHarmonic(FMax, FCount);
    for j:= aMin to aMax do
      FSeries[0].AddXY(aBaseFreq*j,
        Sqrt(Sqr(aData[0,2*j]) + Sqr(aData[0,2*j+1])) /
        Sqrt(Sqr(aData[1,2*j]) + Sqr(aData[1,2*j+1])));
    Exit;
  end;

  for i:= 0 to 1 do if Length(aData[i]) > 0 then
    case FMode of
      smSamples:
        begin
          Inc(FSeriesNumber);
          for j:= 0 to High(aData[i]) do
            FSeries[i].AddXY(1E6*j/aSampling, aData[i,j]);
        end;
       smFFTCom1, smFFTCom2, smFFTMod:
         begin
           aBaseFreq:= aSampling/Length(aData[i]);
           if rgMode.ItemIndex = 0 then begin
             aMin:= FOutput.GetHarmonic(cnMinFreq, FCount);
             aMax:= FOutput.GetHarmonic(cnMaxFreq, FCount);
           end else begin
             aMin:= FOutput.GetHarmonic(FMin, FCount);
             aMax:= FOutput.GetHarmonic(FMax, FCount);
           end;
           case FMode of
             smFFTCom1, smFFTCom2:
               if (i = 0) or (FMode = smFFTCom2) then begin
                 Inc(FSeriesNumber, 2);
                 for j:= aMin to aMax do begin
                   if FAmPh then begin
                     aVal:= cAmPh(cCom(aData[i,2*j]));
                     aVal.y:= RadToDeg(aVal.y); 
                   end else
                     aVal:= cCom(aData[i,2*j]);
                   FSeries[2*i].AddXY(aBaseFreq*j, aVal.x);
                   FSeries[2*i+1].AddXY(aBaseFreq*j, aVal.y);
                 end;
               end;
             smFFTMod:
               begin
                 Inc(FSeriesNumber, 1);
                 for j:= aMin to aMax do
                   FSeries[i].AddXY(aBaseFreq*j,
                     Sqrt(Sqr(aData[i,2*j]) + Sqr(aData[i,2*j+1])));
               end;
           end;
         end;
    end;
end;

procedure TForm1.SpRangeChange(aMin, aMax: Double);
begin
  FOutput.SetRange(aMin, aMax, FAmp, FCount);
  FMin:= aMin;
  FMax:= aMax;
end;

procedure TForm1.OnSamplingChange(aSampling: DWORD);
var
  aMin, aMax: Integer;
begin
  FCount:= 1 shl Round(Log2(frWaveSetup1.Sampling / cnMinFreq));
  aMin:= Trunc(aSampling / FCount);
  if aMin < 1 then aMin:= 1;
  aMax:= aMin*(FCount div 2 - 1);
  if aMax > cnMaxFreq then
    aMax:= cnMaxFreq;
  frSpRange1.SetLimits(aMin, aMax);
end;

procedure TForm1.DataProcess(aData: TWaveData);
begin
{  for i:= 0 to 1 do
    WaveFFT(@aData[i,0], Length(aData[i]), False); }
  case rgMode.ItemIndex of
    1,2: FInput.ProcZLCRMeter(aData, FMin, FMax, frRref.CurrentValue,
       frWaveSetup1.cbSwap.Checked, rgMode.ItemIndex=2, ShowLCMessage);
    3: FInput.Transfer(aData, FMin, FMax, frWaveSetup1.cbSwap.Checked);
  end;
  ShowData(aData);
end;

procedure TForm1.DrawAll(aValue: Boolean);
var
  i: Integer;
begin
  for i:= 0 to 3 do
    FSeries[i].DrawAllPoints:= aValue;
end;

procedure TForm1.LeftForAll(aValue: Boolean);
var
  aAxis: TVertAxis;
begin
  if aValue then
    aAxis:= aLeftAxis
  else
    aAxis:= aRightAxis;
  Series2.VertAxis:= aAxis;
  Series4.VertAxis:= aAxis;
  Chart.LeftAxis.Title.Visible:= not aValue;
end;

procedure TForm1.cbDrawAllClick(Sender: TObject);
begin
  DrawAll(cbDrawAll.Checked);
end;

procedure TForm1.cbLeftForAllClick(Sender: TObject);
begin
  LeftForAll(cbLeftForAll.Checked);
end;

procedure TForm1.cbLogFClick(Sender: TObject);
begin
  Chart.BottomAxis.Logarithmic:= cbLogF.Checked;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
const
  cnMaxTime = 2000; //3 sec
var
  aTime: TDateTime;
  aSt: String;
begin
  SaveIni;
  FInput.Stop;
  FOutput.Stop;
  aTime:= Time;
  repeat
    if FInput.Suspended and FOutput.Suspended then
      Break;
  until DateTimeToTimeStamp(Time - aTime).Time > cnMaxTime;
  aSt:= '';
  if not FInput.Suspended then
    aSt:= aSt + 'Can not stop WaveIn.' + Chr(13)+Chr(10);
  if not FOutput.Suspended then
    aSt:= aSt + 'Can not stop WaveOut.' + Chr(13)+Chr(10);
  if aSt <> '' then begin
    aSt:= aSt + 'Abnormal termination.' + Chr(13)+Chr(10)+
      'Try use Task Manager to close this software.';
    ShowMessage(aSt);
  end;
  CanClose:= True;
end;

procedure TForm1.ShowErrorMessage(const aMessage: String);
begin
  ShowMessage(aMessage);
end;

procedure TForm1.ShowCalibMesssage(const aMessage: String);
begin
  lbCalibration.Caption:= aMessage;
end;

procedure TForm1.ShowLCMessage(const aMessage: String);
begin
  memLCRRes.Text:= aMessage;
end;

procedure TForm1.Saturation(aValue: Boolean);
begin
  lbSaturation.Visible:= aValue;
end;

procedure TForm1.ClearSeries;
var
  i: Integer;
begin
  for i:= 0 to 3 do
    FSeries[i].Clear;
end;

procedure TForm1.btnSaveClick(Sender: TObject);
var
  i: Integer;
  aFile: TextFile;
begin
  AssignFile(aFile, ChangeFileExt(Paramstr(0),'.txt'));
  Rewrite(aFile);
  if rgMode.ItemIndex = 4 then
    for i:= 0 to Series1.XValues.Count - 1 do begin
      WriteLn(aFile, Series1.XValues[i], ' ', Series3.YValues[i], ' ', Series4.YValues[i],
                                         ' ', Series1.YValues[i], ' ', Series2.YValues[i]);
    end
  else
    for i:= 0 to Series1.XValues.Count - 1 do begin
      WriteLn(aFile, Series1.XValues[i], ' ', Series1.YValues[i], ' ', Series2.YValues[i]);
    end;
  CloseFile(aFile);
end;

procedure TForm1.ChartClickAxis(Sender: TCustomChart; Axis: TChartAxis;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Axis = Chart.LeftAxis then begin
    Axis.Grid.Visible:= True;
    Chart.RightAxis.Grid.Visible:= False;
  end;
  if Axis = Chart.RightAxis then begin
    Axis.Grid.Visible:= True;
    Chart.LeftAxis.Grid.Visible:= False;
  end;
end;

procedure TForm1.rgModeClick(Sender: TObject);
begin
  gbCalibration.Visible:= rgMode.ItemIndex = 0;
  gbGraph.Visible:= not gbCalibration.Visible;
  gbLCR.Visible:= rgMode.ItemIndex = 2;
  Chart.Legend.Visible:= gbLCR.Visible;
  case rgMode.ItemIndex of
    1,2: pnRref.Visible:= True;
    else pnRref.Visible:= False;
  end;
  lbCalibration.Caption:= '';
  if rgMode.ItemIndex = 0 then begin
    FInput.Accumulation:= False;
    DrawAll(True);
    frSpRange1.Visible:= False;
  end else begin
    SetGraphParams;
    frSpRange1.Visible:= True;
    FCurMesId:= rgMode.ItemIndex;
  end;
end;

procedure TForm1.cbModPhClick(Sender: TObject);
begin
  FAmPh:= cbModPh.Checked;
  if FAmPh then begin
    Chart.LeftAxis.Title.Caption:= 'Module';
    Chart.RightAxis.Title.Caption:= 'Phase';
    Series1.Title:= 'Module';
    Series3.Title:= '|Zx|';
    Series2.Title:= 'Phase';
    Series4.Title:= 'Phase Zx';
  end else begin
    Chart.LeftAxis.Title.Caption:= 'Re';
    Chart.RightAxis.Title.Caption:= 'Im';
    Series1.Title:= 'Re';
    Series3.Title:= 'R';
    Series2.Title:= 'Im';
    Series4.Title:= 'L/C';
  end;
end;

procedure TForm1.cbIntegrationClick(Sender: TObject);
begin
  FInput.SetAccTime(cbIntegration.Items[cbIntegration.ItemIndex]);
end;

end.
