unit ufrSpRange;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ToolWin, ComCtrls;

type
  TRangeProc = procedure(aMin, aMax: Double) of object;

  TfrSpRange = class(TFrame)
    pnFullRange: TPanel;
    SpMax: TSplitter;
    SpMin: TSplitter;
    pnMin: TPanel;
    pnMax: TPanel;
    pnRange: TPanel;
    pnLine: TPanel;
    imScale: TImage;
    procedure SpMinMoved(Sender: TObject);
    procedure pnRangeMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pnRangeMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pnRangeMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FrameResize(Sender: TObject);
    procedure SpCanResize(Sender: TObject; var NewSize: Integer;
      var Accept: Boolean);
    procedure pnRangeResize(Sender: TObject);
    procedure pnRangeDblClick(Sender: TObject);
    procedure pnFullRangeResize(Sender: TObject);
    procedure SpMaxMoved(Sender: TObject);
  private
    { Private declarations }
    FMouseDown: Boolean;
    FMouseLeft,
    FMouseX: Integer;
    FMinLim, FMaxLim,
    FMin, FMax,
    FMinSav, FMaxSav: Double;
    FRangeProc: TRangeProc;
    procedure OnRangeChange;
    function ScrollLeft: Integer;
    function ScrollRight: Integer;
    procedure FillScale(aMin, aMax: Double);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure SetLimits(aMin, aMax: Double);
    procedure SetRange(aMin, aMax: Double);
    property Min: Double read FMin;
    property Max: Double read FMax;
    property RangeProc: TRangeProc write FRangeProc;
  end;

implementation

{$R *.DFM}

const
  cnPnMinWidth = 2;

function ValToStr(aVal: Double): String;
begin
  //Result:= FloatToStrF(aVal, {ffFixed}ffNumber, 9, 0);
  Result:= Format('%0.0f', [aVal]);
end;

constructor TfrSpRange.Create(AOwner: TComponent);
var
  aBitMap: TBitMap;
begin
  inherited;
  FMouseDown:= False;
  aBitMap:= TBitMap.Create;
  aBitMap.Height:= imScale.Height;
  aBitMap.Width:= imScale.Width;
  imScale.Picture.BitMap:= aBitMap;
  aBitMap.Free;
  FRangeProc:= nil;
  SetLimits(1, 100);
end;

procedure TfrSpRange.SpMinMoved(Sender: TObject);
begin
  if SpMax.Left < pnRange.Left + pnRange.Width then begin
    if pnRange.Left - SpMin.Width - cnPnMinWidth > cnPnMinWidth then
      pnMin.Width:= pnRange.Left - SpMin.Width - cnPnMinWidth;
    pnRange.Width:= 0;
  end;
  OnRangeChange;
end;

procedure TfrSpRange.SpMaxMoved(Sender: TObject);
begin
  if SpMax.Left < pnRange.Left + pnRange.Width then begin
    pnRange.Width:= 0;
  end;
  OnRangeChange;
end;

procedure TfrSpRange.pnRangeMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button <> mbLeft then
    Exit;
  FMouseDown:= True;
  FMouseLeft:= Mouse.CursorPos.x - pnRange.Left;
  FMouseX:= Mouse.CursorPos.x;
  pnRange.Color:= clBtnShadow;
end;

procedure TfrSpRange.pnRangeMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button <> mbLeft then
    Exit;
  FMouseDown:= False;
  pnRange.Color:= clBtnFace;
  if Mouse.CursorPos.x <> FMouseX then
    OnRangeChange;
end;

procedure TfrSpRange.pnRangeMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
var
  aLeft, aWidth, aRangeWidth: Integer;
begin
  if not FMouseDown then
    Exit;
  aRangeWidth:= pnRange.Width;
  aLeft:= Mouse.CursorPos.x - FMouseLeft;
  if aLeft < ScrollLeft then
    aLeft:= ScrollLeft;
  if aLeft + aRangeWidth > ScrollRight then
    aLeft:= ScrollRight - aRangeWidth;
  aWidth:= aLeft - SpMin.Width - 1;
  if aWidth = pnMin.Width then
    Exit;
  pnMin.Width:= aWidth;
  pnMax.Width:= pnFullRange.Width - aLeft - aRangeWidth - SpMax.Width - 1;
  //OnRangeChange;
end;

procedure TfrSpRange.OnRangeChange;
var
  aK: Double;
begin
  aK:= (FMaxLim - FMinLim) / (ScrollRight - ScrollLeft);
  FMin:= FMinLim + (pnRange.Left - ScrollLeft) * aK;
  FMax:= FMinLim + (pnRange.Left + pnRange.Width - ScrollLeft) * aK;
  if (FMin <> FMinLim) or (FMax <> FMaxLim) then begin
    FMinSav:= FMin;
    FMaxSav:= FMax;
  end;  
  if Assigned(FRangeProc) then
    FRangeProc(FMin, FMax);
end;

procedure TfrSpRange.SpCanResize(Sender: TObject; var NewSize: Integer;
  var Accept: Boolean);
begin
  if NewSize < cnPnMinWidth then
    NewSize:= cnPnMinWidth;
  Accept:= True;
end;

procedure TfrSpRange.FrameResize(Sender: TObject);
begin
  SetRange(FMin, FMax);
end;

function TfrSpRange.ScrollLeft: Integer;
begin
  Result:= SpMin.Width + cnPnMinWidth + 1;
end;

function TfrSpRange.ScrollRight: Integer;
begin
  Result:= pnFullRange.Width - SpMax.Width - cnPnMinWidth - 1;
end;

procedure TfrSpRange.SetLimits(aMin, aMax: Double);
begin
  FMinLim:= aMin;
  FMaxLim:= aMax;
  FillScale(aMin, aMax);
  SetRange(aMin, aMax);
  FMinSav:= aMin + (aMax - aMin) * 1/3;
  FMaxSav:= aMin + (aMax - aMin) * 2/3;
end;

procedure TfrSpRange.SetRange(aMin, aMax: Double);
var
  aK: Double;
begin
  aK:= (pnFullRange.Width - 2*cnPnMinWidth - SpMin.Width - SpMax.Width - 2)
     / (FMaxLim - FMinLim);
  if aMin < FMinLim then
    aMin:= FMinLim;
  if aMax > FMaxLim then
    aMax:= FMaxLim;
  FMin:= aMin;
  FMax:= aMax;
  if (FMin <> FMinLim) or (FMax <> FMaxLim) then begin
    FMinSav:= FMin;
    FMaxSav:= FMax;
  end;  
  pnMin.Width:= cnPnMinWidth + Round((FMin - FMinLim) * aK);
  pnMax.Width:= cnPnMinWidth + Round((FMaxLim - FMax) * aK);
  if Assigned(FRangeProc) then
    FRangeProc(FMin, FMax);
end;

procedure TfrSpRange.pnRangeResize(Sender: TObject);
begin
  pnLine.Width:= pnRange.Width;
end;

procedure TfrSpRange.pnRangeDblClick(Sender: TObject);
begin
  if (FMin = FMinLim) and (FMax = FMaxLim) then
    SetRange(FMinSav, FMaxSav)
  else
    SetRange(FMinLim, FMaxLim);
end;

procedure TfrSpRange.pnFullRangeResize(Sender: TObject);
begin
  imScale.Width:= pnFullRange.Width;
  FillScale(FMinLim, FMaxLim);
end;

function IdCoeff(aId: Integer): Integer;
var
  aPow, i: Integer;
begin
  case (aId mod 3) of
    0: Result:= 2;
    1: Result:= 5;
    2: Result:= 10;
  else
    Result:= 1;
  end;
  aPow:= aId div 3;
  for i:= 1 to aPow do
    Result:= Result * 10;
end;

procedure TfrSpRange.FillScale(aMin, aMax: Double);
const
  cnY = 1;
var
  aSt: String;
  aBaseVal, aStep, aTick, aK: Double;
  aMaxNmb, i: Integer;
  aX, aWidth: Integer;

function ValPos(aVal: Double): Integer;
begin
  Result:= ScrollLeft + Round(aK*(aVal-aMin));
end;

procedure ValOut(aVal: Double);
var
  aX: Integer;
begin
  aX:= ValPos(aVal);
  imScale.Canvas.TextOut(aX, cnY, ValToStr(aVal));
  imScale.Canvas.MoveTo(aX, imScale.Canvas.ClipRect.Bottom - 4);
  imScale.Canvas.LineTo(aX, imScale.Canvas.ClipRect.Bottom);
end;

procedure TickOut(aVal: Double);
var
  aX: Integer;
begin
  aX:= ValPos(aVal);
  imScale.Canvas.MoveTo(aX, imScale.Canvas.ClipRect.Bottom - 2);
  imScale.Canvas.LineTo(aX, imScale.Canvas.ClipRect.Bottom);
end;

begin
  imScale.Picture.Graphic.Width:= imScale.Width;
  imScale.Canvas.Brush.Color:= clBtnFace;
  imScale.Canvas.FillRect(imScale.Canvas.ClipRect);
  SetBkMode(imScale.Canvas.Handle, TRANSPARENT);
  aSt:= FloatToStrF(aMax, ffExponent, 1, 1);
  i:= Pos('.', aSt);
  if i = 0 then
    i:= Pos(',', aSt);
  aSt[i-1]:= '1';
  while aSt[i+1] <> 'E' do
    Delete(aSt, i+1, 1);
  aBaseVal:= StrToFloat(aSt);
  aMaxNmb:= imScale.Width div imScale.Canvas.TextWidth(ValToStr(aBaseVal));
  aStep:= aBaseVal;
  i:= 0;
  if Trunc((aMax - aMin)/ aStep) * 2 < aMaxNmb then
    repeat
      aStep:= aBaseVal / IdCoeff(i);
      Inc(i);
    until Trunc((aMax - aMin)*(IdCoeff(i+1)/aBaseVal)) > aMaxNmb
  else
    while Trunc((aMax - aMin)/ aStep) > aMaxNmb do begin
      aStep:= aBaseVal * IdCoeff(i);
      Inc(i);
    end;
  aSt:= FloatToStr(aStep);
  if aSt[1] = '2' then
    aTick:= aStep / 2
  else
    aTick:= aStep / 5;
  aK:= (ScrollRight - ScrollLeft)/(aMax-aMin);
  aWidth:= imScale.Canvas.TextWidth(ValToStr(aBaseVal)+'0');
  if aMin > aBaseVal then
    if Trunc(aMin/aBaseVal) < Trunc(aMax/aBaseVal) then
      aBaseVal:= Trunc(aMax/aBaseVal) else
    begin
      aBaseVal:= Trunc(aMax/aStep)*aStep;
    end;
  // Text labels
  aX:= ValPos(aBaseVal);
  if aX + aWidth < imScale.Width then
    ValOut(aBaseVal);
  i:= 1;
  while ValPos(aBaseVal-i*aStep) > 0 do begin
    ValOut(aBaseVal-i*aStep);
    Inc(i);
  end;
  i:= 1;
  while ValPos(aBaseVal+i*aStep) < imScale.Width - aWidth do begin
    ValOut(aBaseVal+i*aStep);
    Inc(i);
  end;
  // Ticks
  i:= 1;
  while ValPos(aBaseVal-i*aTick) > 0 do begin
    TickOut(aBaseVal-i*aTick);
    Inc(i);
  end;
  i:= 1;
  while ValPos(aBaseVal+i*aTick) < imScale.Width do begin
    TickOut(aBaseVal+i*aTick);
    Inc(i);
  end;
end;

end.
