unit ufrWaveSetup;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, IniFiles;

type
  TProcSampling = procedure(aSampling: dWORD) of object;

  TfrWaveSetup = class(TFrame)
    cbSampling: TComboBox;
    lbSampling: TLabel;
    lbInput: TLabel;
    cbDeviceIn: TComboBox;
    BevelWS1: TBevel;
    lbOutput: TLabel;
    cbDeviceOut: TComboBox;
    BevelWS2: TBevel;
    cbSwap: TCheckBox;
    procedure cbSamplingChange(Sender: TObject);
  private
    { Private declarations }
    FProcSampling: TProcSampling;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    function Sampling: DWORD;
    function DeviceInId: Integer;
    function DeviceOutId: Integer;
    procedure AllowInput(aEnable: Boolean);
    procedure LoadIni(aFile: TIniFile);
    procedure SaveIni(aFile: TIniFile);
    property ProcSampling: TProcSampling write FProcSampling;
  end;

implementation

{$R *.DFM}

uses
  MMSystem;

const
  csSection = 'WaveSetup';  

constructor TfrWaveSetup.Create(AOwner: TComponent);
var
  i: Integer;
  aCaps: WAVEINCAPS;
begin
  inherited;
  cbSampling.ItemIndex:= 0;
  for i:= 0 to cbSampling.Items.Count - 1 do
    cbSampling.Items.Objects[i]:= TObject(StrToInt(cbSampling.Items[i]));
  cbDeviceIn.Items.Clear;
  for i:= 0 to waveInGetNumDevs - 1 do begin
    waveInGetDevCaps(i, @aCaps, SizeOf(aCaps));
    cbDeviceIn.Items.Add(aCaps.szPname);
  end;
  cbDeviceIn.ItemIndex:= waveInGetNumDevs div 2;
  cbDeviceOut.Items.Clear;
  for i:= 0 to waveOutGetNumDevs - 1 do begin
    waveOutGetDevCaps(i, @aCaps, SizeOf(aCaps));
    cbDeviceOut.Items.Add(aCaps.szPname);
  end;
  cbDeviceOut.ItemIndex:= waveOutGetNumDevs div 2;
end;

procedure TfrWaveSetup.LoadIni(aFile: TIniFile);
var
  aInt, i: Integer;
  aSt: String;
begin
  aInt:= aFile.ReadInteger(csSection, 'InputID', cbDeviceIn.ItemIndex);
  if aInt < cbDeviceIn.Items.Count then begin
    cbDeviceIn.ItemIndex:= aInt;
    aSt:= aFile.ReadString(csSection, 'InputName', cbDeviceIn.Items[aInt]);
    if aSt = cbDeviceIn.Items[aInt] then begin
      // Input Mixer load here
    end;
  end;
  aInt:= aFile.ReadInteger(csSection, 'OutputID',
         cbDeviceOut.ItemIndex);
  if aInt < cbDeviceOut.Items.Count then begin
    cbDeviceOut.ItemIndex:= aInt;
    aSt:= aFile.ReadString(csSection, 'OutputName',
          cbDeviceOut.Items[aInt]);
    if aSt = cbDeviceOut.Items[aInt] then begin
      // Output Mixer load here
    end;
  end;
  aInt:= aFile.ReadInteger(csSection, 'Sampling',
         Integer(cbSampling.Items.Objects[cbSampling.ItemIndex]));
  for i:= 0 to cbSampling.Items.Count - 1 do
    if cbSampling.Items.Objects[i] = TObject(aInt) then begin
      cbSampling.ItemIndex:= i;
      Break;
    end;
  cbSwap.Checked:= aFile.ReadBool(csSection, 'Swap', cbSwap.Checked);
end;

procedure TfrWaveSetup.SaveIni(aFile: TIniFile);
begin
  aFile.WriteInteger(csSection, 'InputID', cbDeviceIn.ItemIndex);
  aFile.WriteString(csSection, 'InputName',
                    cbDeviceIn.Items[cbDeviceIn.ItemIndex]);
  // Input Mixer save here
  
  aFile.WriteInteger(csSection, 'OutputID', cbDeviceOut.ItemIndex);
  aFile.WriteString(csSection, 'OutputName',
                    cbDeviceOut.Items[cbDeviceOut.ItemIndex]);
  // Output Mixer save here

  aFile.WriteInteger(csSection, 'Sampling',
                     Integer(cbSampling.Items.Objects[cbSampling.ItemIndex]));
  aFile.WriteBool(csSection, 'Swap', cbSwap.Checked);
end;

function TfrWaveSetup.Sampling: DWORD;
begin
  Result:= Integer(cbSampling.Items.Objects[cbSampling.ItemIndex]);
end;

function TfrWaveSetup.DeviceInId: Integer;
begin
  Result:= cbDeviceIn.ItemIndex;
end;

function TfrWaveSetup.DeviceOutId: Integer;
begin
  Result:= cbDeviceOut.ItemIndex;
end;

procedure TfrWaveSetup.cbSamplingChange(Sender: TObject);
begin
  if Assigned(FProcSampling) then
    FProcSampling(Sampling);
end;

procedure TfrWaveSetup.AllowInput(aEnable: Boolean);
begin
  cbSampling.Enabled:= aEnable;
  cbDeviceIn.Enabled:= aEnable;
  cbDeviceOut.Enabled:= aEnable;
end;

end.
