unit uCom;

interface

uses
  Math;

type
  Rel = Double; // Real value
  Com = record // Complex value
   case Byte of
     0: (x, y: Rel);
     1: (u: array [0..1] of Rel);
   end;

const
  Com0: Com = (x: 0.0; y: 0.0);
  Com1: Com = (x: 1.0; y: 0.0);

function cCom (const a): Com;
function cMod2 (a: Com): Rel;
function cAdd (a, b: Com): Com;
function cMul(a, b: Com): Com;
function cSub (a, b: Com): Com;
function cInv (a: Com): Com;
function cDiv(a, b: Com): Com;
function cMuR (a: Com; b: Rel): Com;
function cCon (a: Com): Com;
function cAmPh (a: Com): Com;
function cCmAP (a: Com): Com;

implementation

// --- Com lib -----------------------------------------------------------------

function cCom (const a): Com;
begin
  Result:= Com(a);
end;

function cMod2 (a: Com): Rel;
begin
  Result:= Sqr(a.x) + Sqr(a.y);
end;

function cAdd (a, b: Com): Com;
var i: Integer;
begin
  for i:= Low(a.u) to High(a.u) do
    Result.u[i]:= a.u[i] + b.u[i];
end;

function cSub (a, b: Com): Com;
var i: Integer;
begin
  for i:= Low(a.u) to High(a.u) do
    Result.u[i]:= a.u[i] - b.u[i];
end;

function cInv (a: Com): Com;
var t, d: Rel;
begin
  Result:= Com0;
  if (a.x = 0) and (a.y = 0) then
    Exit;
  if abs(a.x) >= abs(a.y) then begin
    t:= a.y / a.x;
    d:= a.x + t * a.y;
    Result.x:= 1 / d;
    Result.y:= - t / d;
  end else begin
    t:= a.x / a.y;
    d:= a.y + t * a.x;
    Result.x:=   t / d;
    Result.y:=  -1 / d;
  end;
end;

function cMul(a, b: Com): Com;
begin
  Result.x:= a.x*b.x - a.y*b.y;
  Result.y:= a.x*b.y + a.y*b.x;
end;

function cDiv(a, b: Com): Com;
var t, d: Rel;
begin
  Result:= Com0;
  if ((a.x = 0) and (a.y = 0)) or ((b.x = 0) and (b.y = 0)) then
    Exit;
  if abs(b.x) >= abs(b.y) then begin
    t:= b.y / b.x;
    d:= b.x + t * b.y;
    Result.x:= (a.x + a.y * t) / d;
    Result.y:= (a.y - a.x * t) / d;
  end else begin
    t:= b.x / b.y;
    d:= b.y + t * b.x;
    Result.x:= (a.x * t + a.y) / d;
    Result.y:= (a.y * t - a.x) / d;
  end;
end;

function cMuR (a: Com; b: Rel): Com;
var i: Integer;
begin
  for i:= Low(a.u) to High(a.u) do
    Result.u[i]:= a.u[i] * b;
end;

function cCon (a: Com): Com;
begin
  Result.x:=  a.x;
  Result.y:= -a.y;
end;

function cAmPh (a: Com): Com;
begin
  Result.x:= Sqrt(cMod2(a));
  Result.y:= ArcTan2(a.y, a.x);
end;

function cCmAP (a: Com): Com;
var
  aSi, aCo: Extended;
begin
  SinCos(a.y, aSi, aCo);
  Result.x:= a.x * aCo;
  Result.x:= a.x * aSi;
end;

end.
