unit uFFT;

interface

type
  TWaveArray = array[0..MaxInt div SizeOf(Double) - 1] of Double;
  pWaveMask = ^TWaveArray;

procedure WaveFFT(aV: pWaveMask; aNN: Integer; aFFT : Boolean);

implementation

procedure WaveFFT(aV: pWaveMask; aNN: Integer; aFFT : Boolean);
var
  twr, twi, twpr, twpi, twtemp, ttheta: Double;
  i, i1, i2, i3, i4: Integer;
  c1, c2, h1r, h1i, h2r, h2i, wrs, wis: Double;
  nn, ii, jj, n, mmax, m, j: Integer;
  istep, isign: Integer;
  wtemp, wr, wpr, wpi, wi, theta, tempr, tempi: Double;
begin
  if aNN = 1 then
    Exit;
  ttheta:= 2*Pi/aNN;
  c1:= 0.5;
  if aFFT then begin
    c2:= -0.5;
  end else begin
    c2:= 0.5;
    ttheta:= -ttheta;
    twpr:= -2.0*sqr(sin(0.5*ttheta));
    twpi:= sin(ttheta);
    twr:= 1.0+twpr;
    twi:= twpi;
    i:=2;
    while i<=aNN div 4+1 do begin
      i1:= i+i-2;
      i2:= i1+1;
      i3:= aNN+1-i2;
      i4:= i3+1;
      wrs:= twr;
      wis:= twi;
      h1r:=  c1*(aV[i1]+aV[i3]);
      h1i:=  c1*(aV[i2]-aV[i4]);
      h2r:= -c2*(aV[i2]+aV[i4]);
      h2i:=  c2*(aV[i1]-aV[i3]);
      aV[i1]:=  h1r+wrs*h2r-wis*h2i;
      aV[i2]:=  h1i+wrs*h2i+wis*h2r;
      aV[i3]:=  h1r-wrs*h2r+wis*h2i;
      aV[i4]:= -h1i+wrs*h2i+wis*h2r;
      twtemp:= twr;
      twr:= twr*twpr-twi*twpi+twr;
      twi:= twi*twpr+twtemp*twpi+twi;
      Inc(i);
    end;
    h1r:= aV[0];
    aV[0]:= c1*(h1r+aV[1]);
    aV[1]:= c1*(h1r-aV[1]);
  end;
  if aFFT then
    isign:= 1
  else
    isign:= -1;
  n:= aNN;
  nn:= aNN div 2;
  j:= 1;
  ii:=1;
  while ii<=nn do begin
    i:= 2*ii-1;
    if j>i then begin
      tempr:= aV[j-1];
      tempi:= aV[j];
      aV[j-1]:= aV[i-1];
      aV[j]:= aV[i];
      aV[i-1]:= tempr;
      aV[i]:= tempi;
    end;
    m:= n div 2;
    while (m>=2) and (j>m) do begin
      j:= j-m;
      m:= m div 2;
    end;
    j:= j+m;
    Inc(ii);
  end;
  mmax:= 2;
  while n>mmax do begin
    istep:= 2*mmax;
    theta:= 2*Pi/(isign*mmax);
    wpr:= -2.0*sqr(sin(0.5*theta));
    wpi:= sin(theta);
    wr:= 1.0;
    wi:= 0.0;
    ii:=1;
    while ii<=mmax div 2 do begin
      m:= 2*ii-1;
      jj:=0;
      while jj<=(n-m) div istep do begin
        i:= m+jj*istep;
        j:= i+mmax;
        tempr:= wr*aV[j-1]-wi*aV[j];
        tempi:= wr*aV[j]+wi*aV[j-1];
        aV[j-1]:= aV[i-1]-tempr;
        aV[j]:= aV[i]-tempi;
        aV[i-1]:= aV[i-1]+tempr;
        aV[i]:= aV[i]+tempi;
        Inc(jj);
      end;
      wtemp:= wr;
      wr:= wr*wpr-wi*wpi+wr;
      wi:= wi*wpr+wtemp*wpi+wi;
      Inc(ii);
    end;
    mmax:= istep;
  end;
  if not aFFT then begin
    i:=1;
    while I<=2*nn do begin
      aV[I-1]:= aV[i-1]/nn;
      Inc(i);
    end;
  end else begin
    twpr:= -2.0*sqr(sin(0.5*ttheta));
    twpi:= sin(ttheta);
    twr:= 1.0+twpr;
    twi:= twpi;
    i:= 2;
    while i<=aNN div 4+1 do begin
      i1:= i+i-2;
      i2:= i1+1;
      i3:= aNN+1-i2;
      i4:= i3+1;
      wrs:= twr;
      wis:= twi;
      h1r:= c1*(aV[i1]+aV[i3]);
      h1i:= c1*(aV[i2]-aV[i4]);
      h2r:= -c2*(aV[i2]+aV[i4]);
      h2i:= c2*(aV[i1]-aV[i3]);
      aV[i1]:= h1r+wrs*h2r-wis*h2i;
      aV[i2]:= h1i+wrs*h2i+wis*h2r;
      aV[i3]:= h1r-wrs*h2r+wis*h2i;
      aV[i4]:= -h1i+wrs*h2i+wis*h2r;
      twtemp:= twr;
      twr:= twr*twpr-twi*twpi+twr;
      twi:= twi*twpr+twtemp*twpi+twi;
      Inc(i);
    end;
    h1r:= aV[0];
    aV[0]:= h1r+aV[1];
    aV[1]:= h1r-aV[1];
  end;
end;


end.
